local addonName = ...

local DRESSUPFRAME = DressUpFrame
local DRESSUPMODEL = DressUpModel
local DRESSUPRESET = DressUpFrameResetButton

local GetInventoryItemLink = GetInventoryItemLink
local NotifyInspect = NotifyInspect
local PlaySound = PlaySound
local UnitExists = UnitExists
local UnitGUID = UnitGUID
local UnitIsPlayer = UnitIsPlayer

local PLAYER = "player"
local TARGET = "target"

local buttons = {}

local function getFreeSlot()
  for i = #buttons, 1, -1 do
    return buttons[i]
  end
  return DRESSUPRESET
end

local function addButton(meta)
  if type(meta) ~= "table" then
    return
  end
  if not meta.title then
    return
  end
  meta.size = meta.size or {42, 22}
  local button = CreateFrame("Button", addonName.."Button"..(#buttons + 1), DRESSUPFRAME, "UIPanelButtonTemplate")
  button:SetPoint("RIGHT", getFreeSlot(), "LEFT", 0, 0)
  button:SetSize(unpack(meta.size))
  if meta.onupdate then
    button:SetScript("OnUpdate", meta.onupdate)
  end
  if meta.onclick then
    button:SetScript("OnClick", meta.onclick)
  end
  if meta.tooltip then
    button.tooltipText = meta.tooltip
    button.newbieText = meta.newbietip
  end
  button:RegisterForClicks("LeftButtonUp", "RightButtonUp")
  button:SetText(meta.title)
  table.insert(buttons, button)
end

local function TargetExists(button)
  local exists, enabled = UnitExists(TARGET), button:IsEnabled()
  if exists and (not enabled) then
    button:Enable()
  elseif (not exists) and enabled then
    button:Disable()
  end
end

local function TargetIsPlayer(button)
  local player, enabled = UnitIsPlayer(TARGET), button:IsEnabled()
  if player and (not enabled) then
    button:Enable()
  elseif (not player) and enabled then
    button:Disable()
  end
end

local function addTooltip(button, tooltip, newbietip)
  button.tooltipText = tooltip
  button.newbieText = newbietip
  button:HookScript("OnEnter", function(button)
    if button.tooltipText ~= nil then
      GameTooltip_AddNewbieTip(button, button.tooltipText, 1, 1, 1, button.newbieText)
    end
  end)
  button:HookScript("OnLeave", function(button)
    if button.tooltipText ~= nil then
      GameTooltip:Hide()
    end
  end)
end

local function inspectReady(button, event, guid)
  if button.guid == guid then
    local item
    for i = 0, 19 do
      if (not button.skip) or (button.skip and (not (i == 1 or i == 15 or i == 19))) then
        item = GetInventoryItemLink(TARGET, i)
        if item then
          DRESSUPMODEL:TryOn(item)
        end
      end
    end
    for i = 16, 17 do
      item = GetInventoryItemLink(TARGET, i)
      if item then
        DRESSUPMODEL:TryOn(item)
      end
    end
  end
end

local addButtons = {
  {
    title = "P",
    tooltip = "Player character",
    onclick = function()
      DRESSUPMODEL:SetUnit(PLAYER)
      PlaySound("gsTitleOptionOK")
    end,
  },
  {
    title = "T",
    tooltip = "Target model",
    onupdate = TargetExists,
    onclick = function()
      if UnitIsPlayer(TARGET) then
        DRESSUPMODEL:SetUnit(TARGET)
      elseif UnitExists(TARGET) then
        local guid = UnitGUID(TARGET)
        if (guid or ""):len() > 7 then
          DRESSUPMODEL:SetCreature(tonumber(guid:sub(7, 10), 16))
        else
          DRESSUPMODEL:SetUnit(TARGET)
        end
      end
      PlaySound("gsTitleOptionOK")
    end,
  },
  {
    title = "TG",
    tooltip = "Target gear",
    newbietip = "Target gear\n> Left-click to equip all items worn\n> Right-click to skip head, shirt and tabard",
    onupdate = TargetIsPlayer,
    onclick = function(button, mouse)
      DRESSUPMODEL:Undress()
      button.skip = mouse == "RightButton"
      button.guid = UnitGUID(TARGET)
      button:SetScript("OnEvent", inspectReady)
      button:RegisterEvent("INSPECT_READY")
      NotifyInspect(TARGET)
      PlaySound("gsTitleOptionOK")
    end,
  },
  {
    title = "U",
    tooltip = "Undress unit",
    onclick = function()
      DRESSUPMODEL:Undress()
      PlaySound("gsTitleOptionOK")
    end,
  },
}

addTooltip(DRESSUPRESET, "Reset current model")

for i = 1, #addButtons do
  addButton(addButtons[i])
end
